from ete3 import TreeStyle, TextFace, CircleFace,  NodeStyle, add_face_to_node

from haptk import utils

def circle_tree_style():
    ts = TreeStyle()

    ts.branch_vertical_margin = 10
    ts.show_leaf_name = False
    ts.show_scale = False

    ts.mode = "c"
    ts.arc_start = 0 # 0 degrees = 15:00 o'clock
    ts.arc_span = 355
    ts.min_leaf_separation = 0
    ts.root_opening_factor = 0
    ts.optimal_scale_level = "mid"
    ts.allow_face_overlap = True

    return ts



def draw_circle_tree(hst, t, samples_to_tag, colors, branch_point_size, branch_length_as_majority):
    style = NodeStyle()
    style["size"] = 0

    narea_color = "#DAF7A6"
    larea_color = "#FFB6C1"

    # spent_parents = []

    for n in t.traverse():
        n.set_style(style)
        
        indexes = hst.get_node_indexes(n.name)
        if not n.is_leaf():
            label = len(indexes)
            F = TextFace(label, tight_text=True, penwidth=30, fsize=8)
            F.rotation = 270
            n.add_face(F, column=0)

            # text_face = TextFace("", fsize=30, tight_text=True, fgcolor="red")
            # utils.tag_branching_point_to_node(hst, n, text_face, branch_point_size)

        if n.is_leaf():
            if samples_to_tag == []:
                n.set_style(utils.return_node_style("#000", 0))
                n.img_style["bgcolor"] = "#FFF"

            for (color_i, sample_list) in enumerate(samples_to_tag):
                results = [i for i in indexes if hst.get_sample_name(i) in sample_list]
                if results:
                    n.set_style(utils.return_node_style(colors[color_i], 0))
                    n.img_style["bgcolor"] = colors[color_i]

            # text_face = TextFace("", fsize=30, fgcolor="white")
            # spent_parents = utils.tag_big_branch_to_node(hst, n, spent_parents, text_face, branch_length_as_majority)

    return t




import unittest
import haptk

class testGraphs(unittest.TestCase):
    def init(self):
        path = "tests/data/bhst.hst.gz"
        hst = haptk.read_hst(path)
        match_hst = haptk.read_hst(path)
        return hst, match_hst

    def test_index_tree(self):
        hst, _ = self.init()
        hst.index_tree("tests/results/index-tree.png")

    def test_match_tree(self):
        match_hst, hst = self.init()
        match_hst.match_tree(hst, "tests/results/match-tree.png")

    def test_circle_tree(self):
        hst, _ = self.init()
        hst.circle_tree("tests/results/circle-tree.png")

    def test_normal_tree(self):
        hst, _ = self.init()
        hst.normal_tree("tests/results/normal-tree.png")
        

if __name__ == '__main__':
    unittest.main()





import itertools

def find_leaf_neighbors(hst):
    neighbor_sets = []
    for idx in hst.node_indices():
        if len(hst.out_edges(idx)) == 0:

            is_paired_leaf = True
            neighbors = [idx]

            parent = hst.adj_direction(idx, True)

            for k in parent:
                for neighbor in hst.adj_direction(k, False):
                    if neighbor not in neighbors:
                        data = hst.get_node_data(neighbor)
                        # not a leaf node with only 1 sample neighbors
                        if len(data['indexes']) == 1:
                            neighbors.append(neighbor)
                            pass
                        else:
                            is_paired_leaf = False

            if is_paired_leaf:
                neighbor_sets.append(neighbors)

            neighbors.sort()
                
    neighbor_sets.sort()
    neighbor_sets = list(k for k,_ in itertools.groupby(neighbor_sets))
    return neighbor_sets
   



from ete3 import TreeStyle, TextFace, CircleFace,  NodeStyle, add_face_to_node

from haptk import utils

def match_tree_style():
    ts = TreeStyle()
    ts.rotation = 90
    ts.show_leaf_name = False
    ts.show_scale = False
    ts.min_leaf_separation = 0
    ts.branch_vertical_margin = 10
    ts.optimal_scale_level = "full"
    ts.allow_face_overlap = True
    return ts

def circle_match_tree_style():
    ts = TreeStyle()

    ts.branch_vertical_margin = 10
    ts.show_leaf_name = False
    ts.show_scale = False

    ts.mode = "c"
    ts.arc_start = 0 # 0 degrees = 15:00 o'clock
    ts.arc_span = 355
    ts.min_leaf_separation = 0
    ts.root_opening_factor = 0
    ts.optimal_scale_level = "mid"
    ts.allow_face_overlap = True

    return ts

def draw_match_tree(hst, t, output, samples, samples_to_tag, colors, proportions, circle):
    style = NodeStyle()
    style["size"] = 0

    if circle:
        for n in t.traverse():
            n.set_style(style)
        
            indexes = hst.get_node_indexes(n.name)
            if not n.is_leaf():

                if n.name == '0':
                    label = "ROOT"
                else:
                    label = len(indexes)

                F = TextFace(label, tight_text=True, penwidth=30, fsize=8)
                F.rotation = 270
                n.add_face(F, column=0)

            if n.is_leaf():
                if samples_to_tag == []:
                    n.set_style(utils.return_node_style("#000", 0))
                    n.img_style["bgcolor"] = "#FFF"

                for (color_i, sample_list) in enumerate(samples_to_tag):
                    results = [i for i in indexes if hst.samples.get_sample_name(i) in sample_list]
                    if results:
                        n.set_style(utils.return_node_style(colors[color_i], 0))
                        n.img_style["bgcolor"] = colors[color_i]

    else:
        for n in t.traverse():
            n.set_style(style)

            indexes = hst.get_node_indexes(n.name)
            if n.name == '0':
                label = "ROOT"
            else:
                label = len(indexes)

                if proportions:
                    prc = label / len(samples)
                    label = f"{label} ({prc:.2f})"

            F = TextFace(label, tight_text=True, penwidth=30)
            F.rotation = 270
            n.add_face(F, column=0)

            if n.is_leaf():

                if samples_to_tag == []:
                    n.set_style(utils.return_node_style("#000", 0))
                    n.img_style["bgcolor"] = "#FFF"

                for (color_i, sample_list) in enumerate(samples_to_tag):
                    results = [i for i in indexes if hst.get_sample_name(i) in sample_list]
                    if results:
                        n.set_style(utils.return_node_style(colors[color_i], 0))
                        n.img_style["bgcolor"] = colors[color_i]


    return t





from ete3 import TreeStyle, TextFace, CircleFace,  NodeStyle, add_face_to_node
from scipy.stats import fisher_exact

from haptk import utils

def iterate_tree_style():
    ts = TreeStyle()

    ts.branch_vertical_margin = 10
    ts.show_leaf_name = False
    ts.show_scale = False

    ts.mode = "c"
    ts.arc_start = 0 # 0 degrees = 15:00 o'clock
    ts.arc_span = 355
    ts.min_leaf_separation = 0
    ts.root_opening_factor = 0
    ts.optimal_scale_level = "mid"
    ts.allow_face_overlap = False
    # ts.allow_face_overlap = False

    return ts

def iterate_tree_inner(hst, t, samples_to_tag, df, optimizer):
    style = NodeStyle()
    style["size"] = 0

    for n in t.traverse():
        n.set_style(style)
        
        F = optimizer(hst, n, df, samples_to_tag)      

        n.add_face(F, column=0)
        

    return t




from ete3 import Tree, TreeStyle, TextFace, NodeStyle
import json
import argparse
import rustworkx as rx
import gzip
import matplotlib as mpl
import numpy as np

from bisect import bisect_left, bisect_right

def find_gt(a, x):
    'Find leftmost value greater than x'
    i = bisect_right(a, x)
    if i != len(a):
        return a[i]
    raise ValueError

def find_lt(a, x):
    'Find rightmost value less than x'
    i = bisect_left(a, x)
    if i:
        return a[i-1]
    raise ValueError

def find_eq(a, x):
    'Locate the leftmost value exactly equal to x'
    i = bisect_left(a, x)
    if i != len(a) and a[i] == x:
        return i
    raise ValueError


def color_fader(c1,c2,mix): #fade (linear interpolate) from color c1 (at mix=0) to c2 (mix=1)
    c1=np.array(mpl.colors.to_rgb(c1))
    c2=np.array(mpl.colors.to_rgb(c2))
    return mpl.colors.to_hex((1-mix)*c1 + mix*c2)

def tag_branching_point_to_node(hst, n, text_face, branch_sizes):
    count = 0
    for i in hst.get_children_idx_amounts(n):
        if i > branch_sizes:
            count += 1

    if count > 1:
        haplotype = hst.get_node_haplotype(n.name)
        text_face.text = f"{len(haplotype)}"
        n.add_face(text_face, column=0)

def tag_big_branch_to_node(hst, n, spent_parents, text_face, branch_node_amount):
    if n.up:
        parent = n.up
        parent_indexes = hst.get_node_indexes(parent.name)

        if len(parent_indexes) == 2:
            if parent in spent_parents:
                if hst.is_maj_up_to(n, branch_node_amount):
                    # print(f"{n.name} is_maj")

                    haplotype = hst.get_node_haplotype(n.name)
                    start = hst.get_node_start(n.name)
                    stop = hst.get_node_stop(n.name)
                    # print(start, stop)
                    bp = stop['pos'] - start['pos']
                    # text_face.text = f"{len(haplotype)}, {bp}"
                    text_face.text = f"{len(haplotype)}"
                    # F.margin_left = 50
                    n.add_face(text_face, column=0)

                    # n.set_style(utils.return_node_style("white", 0))
                    # n.img_style["bgcolor"] = "white"
            else:
                spent_parents.append(parent)
    return spent_parents

def return_node_style(color, size):
    style = NodeStyle()
    style["fgcolor"] = color
    style["size"] = size
    style["hz_line_type"] = 0
    style["vt_line_type"] = 0
    style["shape"] = "square"
    return style


def hst_to_graph_dict(hst):
    graph_dict = {}
    for node_idx in hst.node_indices():
        neighbors = hst.neighbors(node_idx)
        if not len(neighbors) == 0:
            children_dict = {}

            for nbr_idx in neighbors:
                children_dict[nbr_idx] = hst.get_node_data(nbr_idx)

            graph_dict[node_idx] = children_dict
    return graph_dict

# Originally from https://stackoverflow.com/questions/50003007/how-to-convert-python-dictionary-to-newick-form-format
def newickify(hst, graph_dict, min_samples, hard_cut, min_start, max_stop, right_up) -> str:
    visited_nodes = set()

    # Recursion
    def newick_render_node(node_idx, node_data) -> str:
        assert node_idx not in visited_nodes, "Error: The tree may not be circular!"

        start_stop_clause = False

        if min_start or max_stop:
            # parents = hst.G.predecessors(node_idx)
            # if len(parents) > 0:
                # parent_node = parents[0]
                # start = hst.coords[parent_node['start_idx']]['pos']
                # stop = hst.coords[parent_node['stop_idx']]['pos']
                # print(start, stop, len(parent_node['haplotype']), min_start, max_stop)
            start = node_data['start']['pos']
            stop = node_data['stop']['pos']

            # print(start, stop, len(node_data['haplotype']), min_start, max_stop)

            if min_start and start <= min_start:
                start_stop_clause = True

            if max_stop and stop >= max_stop:
                start_stop_clause = True              

        if node_idx not in graph_dict or len(node_data["indexes"]) < min_samples or start_stop_clause:
            # Leaf node string
            return F'{node_idx}:0'
        else:
            # Add node to visited nodes
            visited_nodes.add(node_idx)

            # Get children
            children = graph_dict[node_idx]

            # Sort keys by size
            keys_and_size = [(child, len(hst.G.get_node_data(child)["indexes"])) for child in children.keys()]
            if right_up:
                keys_and_size.sort(key = lambda x: x[1], reverse=True)
            else:
                keys_and_size.sort(key = lambda x: x[1], reverse=False)

            if hard_cut:
                keys_and_size = list(filter(lambda x: x[1] >= min_samples, keys_and_size))

                if len(keys_and_size) == 0:
                    return F'{node_idx}:0'

            # Children strings recursion
            children_strings = [newick_render_node(child, children[child]) for (child, _) in keys_and_size]
            children_strings = ",".join(children_strings)

            # Parent node string
            return F'({children_strings}){node_idx}:0'

    # Root node is 0
    root_node = 0
    node_data = hst.G.get_node_data(root_node)
    newick_string = newick_render_node(root_node, node_data) + ';'

    # assert visited_nodes == set(graph_dict.keys()), "Error: some nodes aren't in the tree"

    return newick_string




from ete3 import TreeStyle, TextFace, NodeStyle

def index_tree_style():
    ts = TreeStyle()
    ts.rotation = 90
    ts.show_leaf_name = False
    ts.show_scale = False
    ts.min_leaf_separation = 0
    ts.branch_vertical_margin = 5
    ts.optimal_scale_level = "full"
    ts.allow_face_overlap = True
    return ts
    

def draw_index_tree(t, output):
    style = NodeStyle()
    style["size"] = 0

    lstyle = NodeStyle()
    lstyle["fgcolor"] = "#000"
    lstyle["shape"] = "square"
    lstyle["size"] = 1
    lstyle["hz_line_type"] = 0
    lstyle["vt_line_type"] = 0

    for n in t.traverse():
        n.set_style(style)

        label = n.name

        F = TextFace(label, tight_text=True, penwidth=30)
        F.rotation = 270
        n.add_face(F, column=0)
        n.set_style(lstyle)

    return t



import unittest
import haptk

class testLeafMethods(unittest.TestCase):
    def init(self):
        path = "tests/data/bhst.hst.gz"
        hst = haptk.read_hst(path)
        return hst

    def test_leaf_neighbors(self):
        hst = self.init()
        neighbors = hst.leaf_neighbors()

        self.assertEqual(neighbors, [[44, 45], [52, 53]])

    def test_leaf_nodes(self):
        hst = self.init()
        leaf_nodes = hst.leaf_nodes()

        self.assertEqual(leaf_nodes, [3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31, 33, 35, 37, 39, 41, 43, 44, 45, 47, 49, 51, 52, 53])
       
    def test_nodes_data_on_leaf_nodes(self):
        hst = self.init()
        leaf_nodes = hst.leaf_nodes()
        data = hst.get_nodes_data(leaf_nodes)
        # print(data)
        # for d in data:
            # print("nmarkers", len(d['haplotype']))

    def test_node_samples(self):
        hst = self.init()
        leaf_nodes = hst.leaf_nodes()
        samples = hst.node_samples(leaf_nodes)
        correct_samples = [
            ['SAMPLE1'], ['SAMPLE2'], ['SAMPLE14'], ['SAMPLE3'], ['SAMPLE1'], ['SAMPLE4'],
            ['SAMPLE2'], ['SAMPLE5'], ['SAMPLE3'], ['SAMPLE6'], ['SAMPLE4'], ['SAMPLE7'],
            ['SAMPLE5'], ['SAMPLE8'], ['SAMPLE6'], ['SAMPLE9'], ['SAMPLE7'], ['SAMPLE10'],
            ['SAMPLE8'], ['SAMPLE11'], ['SAMPLE9'], ['SAMPLE13'], ['SAMPLE12'], ['SAMPLE10'],
            ['SAMPLE11'], ['SAMPLE12'], ['SAMPLE14'], ['SAMPLE13']
        ]
        self.assertEqual(samples, correct_samples)
        


if __name__ == '__main__':
    unittest.main()




from ete3 import Tree, TreeStyle, TextFace, NodeStyle
import json
import argparse
import rustworkx as rx
import gzip
import math

from haptk import utils
from haptk.circle_tree import draw_circle_tree, circle_tree_style
from haptk.index_tree import draw_index_tree, index_tree_style
from haptk.normal_tree import draw_normal_tree, normal_tree_style
from haptk.match_tree import circle_match_tree_style, draw_match_tree, match_tree_style
from haptk.leaf_neighbors import find_leaf_neighbors
from haptk.iterate_tree import iterate_tree_inner, iterate_tree_style

class HST:
    def __init__(self, G, coords, samples, metadata):
        self.G = G
        self.coords = coords
        self.samples = samples
        self.metadata = metadata

    def get_node_data(self, idx):
        idx = int(idx)
        return self.G.get_node_data(idx)

    def get_node_haplotype(self, idx):
        idx = int(idx)
        data = self.G.get_node_data(idx)
        return data["haplotype"]

    def get_node_indexes(self, idx):
        idx = int(idx)
        data = self.G.get_node_data(idx)
        return data["indexes"]

    def get_node_start(self, idx):
        idx = int(idx)
        data = self.G.get_node_data(idx)
        return data["start"]

    def get_node_stop(self, idx):
        idx = int(idx)
        data = self.G.get_node_data(idx)
        return data["stop"]
    
    def leaf_neighbors(self): 
        return find_leaf_neighbors(self.G)

    def leaf_nodes(self):
        leaf_nodes = []

        for idx in self.G.node_indices():
            if len(self.G.out_edges(idx)) == 0:
                leaf_nodes.append(idx)

        return leaf_nodes

    def node_samples(self, nodes):
        list_of_samples = []

        for node in nodes:
            data = self.G.get_node_data(node)
            node_samples = []

            for index in data['indexes']:
                node_samples.append(self.get_sample_name(index))

            list_of_samples.append(node_samples)
        return list_of_samples

    def get_sample_name(self, index):
        # This division always results in whole numbers on the Rust side
        return self.samples[int(index / self.ploidy_int())]

    def ploidy_int(self):
        match self.metadata['ploidy']:
            case "Mixed":
                return 1
            case "Haploid":
                return 1
            case "Diploid":
                return 2
        
    
    def get_nodes_data(self, nodes):
        data = []

        for node in nodes:
            data.append(self.G.get_node_data(node))

        return data

    def haplotype_string(self, node_data, size):
        start = Coord.new(node_data['start'])
        start_idx = utils.find_eq(self.coords, start)

        haplotype = [self.coords[start_idx + i].get_genotype(gt) for i, gt in enumerate(node_data['haplotype'])]

        if len(haplotype) > size:
            between_len = len(haplotype) - size - 1
            if between_len < 2:
                ht_string = '-'.join(str(x) for x in haplotype)

                return f'{ht_string}'
            
            if self.metadata['hst_type'] == 'UhstLeft':
                ending = haplotype[-size:]
                ending = '-'.join(str(x) for x in ending)

                return f'{haplotype[0]}..({between_len})..{ending}'

            elif self.metadata['hst_type'] == 'UhstRight':
                beginning = haplotype[0:size]
                beginning = '-'.join(str(x) for x in beginning)

                return f'{beginning}..({between_len})..{haplotype[-1]}'

            elif self.metadata['hst_type'] == 'Bhst':
                left = int(math.floor(size/2))
                right = int(math.floor(size/2))
                between_len = len(haplotype) - left - right

                beginning = haplotype[0:left]
                ending = haplotype[-right:]
                beginning = '-'.join(str(x) for x in beginning)
                ending = '-'.join(str(x) for x in ending)

                return f'{beginning}..({between_len})..{ending}'

            else:
                print(f"hst_type {self.metadata['hst_type']} is not supported")
        else:
            haplotype_string = '-'.join(str(x) for x in haplotype)

            return f'{haplotype_string}'


    def is_maj_up_to(self, node, value):
        iter_node = node
        # print(f"checking {iter_node.name} for majority")
        for _ in range(0, value):
            if iter_node.name == "0":
                return True

            if not iter_node.up.name == "0":
                # print("input node:", iter_node.up)
                is_big = self.is_majority_node(iter_node.up)
                if is_big == False:
                    # print(f"run {_}: {iter_node.up.name} is not big")
                    return False

            iter_node = iter_node.up
            
        return True

    def is_majority_node(self, node):
        parent = node.up
        input_node_data = self.get_node_data(int(node.name))
        value = len(input_node_data["indexes"])
        # print(f"input node {node.name} has {value} indexes")

        siblings = {}
        for child in parent.children:
            data = self.get_node_data(int(child.name))
            if not child.name == node.name:
                siblings[child.name] = len(data["indexes"])


        # print(value, siblings.values())
        for v in siblings.values():
            if value < v: 
                return False
  

    def get_children_idx_amounts(self, node):
        return [len(self.get_node_data(c.name)["indexes"]) for c in node.children]

    def iterate_tree(self, df, optimizer, output, to_tag=[], min_size=1, hard_cut=False, min_start=None, max_stop=None, right_up=False, tree_style = iterate_tree_style(), dpi=600, w=1624, h=1624):
        t = create_ete3_tree(self, min_size, hard_cut, min_start, max_stop, right_up)
        t = iterate_tree_inner(self, t, to_tag, df, optimizer)
        t.render(output, units="px", w=w, h=h, tree_style=tree_style, dpi=dpi)

    def match_tree(self, other_hst, output, to_tag=[], colors = ["#ff0000", "#FF69B4", "#4cfe92", "#4ccbfe", "#c9efff", "orange", "yellow"], min_size=1, hard_cut=False, min_start=None, max_stop=None, proportions=False, dpi=600, tree_style=match_tree_style(), circle=False): 
        other_t = create_ete3_tree(other_hst, min_size, hard_cut, min_start, max_stop, right_up=False)
        t = draw_match_tree(self, other_t, output, self.samples, to_tag, colors, proportions, circle)

        if circle:
            tree_style=circle_match_tree_style()

        t.render(output, units="px", tree_style=tree_style, dpi=dpi)

    def index_tree(self, output, min_size=1, hard_cut=False, min_start=None, max_stop=None, dpi=600, tree_style=index_tree_style()): 
        t = create_ete3_tree(self, min_size, hard_cut, min_start, max_stop, right_up=False)
        t = draw_index_tree(t, output)

        t.render(output, units="px", tree_style=tree_style, dpi=dpi)

    def circle_tree(self, output, to_tag=[], colors = ["#ff0000", "#FF69B4", "#4cfe92", "#4ccbfe", "#c9efff", "orange", "yellow"], min_size=1, hard_cut=False, min_start=None, max_stop=None, w=1624, h=1624, dpi=600, tree_style=circle_tree_style(), branch_length_as_majority = 999999999, branch_point_size = 999999999): 
        if len(to_tag) > len(colors):
            raise ValueError("more samples to tag than available colors")

        t = create_ete3_tree(self, min_size, hard_cut, min_start, max_stop, right_up=False)
        t = draw_circle_tree(self, t, to_tag, colors, branch_point_size, branch_length_as_majority)

        t.render(output, w=w, h=h, units="px", tree_style=tree_style, dpi=dpi)

    def normal_tree(self, output, to_tag=[], colors = ["#ff0000", "#FF69B4", "#4cfe92", "#4ccbfe", "#c9efff", "orange", "yellow"], min_size=1, hard_cut=False, min_start=None, max_stop=None, proportions=False, dpi=600, tree_style=normal_tree_style(), show_haplotype=False, n_markers=3, show_pos=False, branch_length_as_majority = 999999999, branch_point_size = 999999999): 
        if len(to_tag) > len(colors):
            raise ValueError("more samples to tag than available colors")

        t = create_ete3_tree(self, min_size, hard_cut, min_start, max_stop, right_up=False)
        t = draw_normal_tree(self, t, to_tag, colors, proportions, show_haplotype, n_markers, show_pos, branch_point_size, branch_length_as_majority)

        t.render(output, units="px", tree_style=tree_style, dpi=dpi)

class Coord:
    def __init__(self, contig, pos, ref, alt):
        self.contig = contig
        self.ref = ref
        self.alt = alt
        self.pos = pos

    def new(dct):
        return Coord(dct['contig'], dct['pos'], dct['ref'], dct['alt'])

    def get_genotype(self, gt):
        if gt == 0:
            return self.ref
        if gt == 1:
            return self.alt
        else:
            raise ValueError

    def __repr__(self):
        return f'{self.contig}_{self.pos}_{self.ref}_{self.alt}'

    def __lt__(self, other):
        # return self.pos < other.pos
        if self.pos < other.pos:
            return True

        if self.pos == other.pos:
            if self.ref < other.ref:
                return True
            elif self.ref == other.ref:
                return self.alt < other.alt

        return False


    def __gt__(self, other):
        # return self.pos > other.pos
        if self.pos > other.pos:
            return True

        if self.pos == other.pos:
            if self.ref > other.ref:
                return True
            elif self.ref == other.ref:
                return self.alt > other.alt

        return False

    def __eq__(self, other):
        return self.pos == other.pos and self.ref == other.ref and self.alt == other.alt
      
def read_hst(path):
    with gzip.open(path, 'rt') as f:
        json_bytes = f.read()
        hst_struct = json.loads(json_bytes)
        hst = hst_struct["hst"]
        metadata = hst_struct["metadata"]
        coords = [Coord(dct['contig'], dct['pos'], dct['ref'], dct['alt']) for dct in metadata["coords"]]
        samples = metadata["samples"]
        del metadata["coords"]
        del metadata["samples"]

        G = rx.PyDiGraph()

        edges_tuple_vec = []
        for edges in hst["edges"]:
            edges_tuple_vec.append((edges[0], edges[1], edges[2]))

        G.add_nodes_from(hst["nodes"])
        G.add_edges_from(edges_tuple_vec)

        # if metadata["selection"] == "All":
            # samples = [val for val in samples for _ in (0, 1)]

        hst = HST(G, coords, samples, metadata)

        return hst

def create_ete3_tree(hst, min_size, hard_cut, min_start, max_stop, right_up):
    graph_dict = utils.hst_to_graph_dict(hst.G)
    newick = utils.newickify(hst, graph_dict, min_size, hard_cut, min_start, max_stop, right_up)
    return Tree(newick, format=1)



        
    





from ete3 import TreeStyle, TextFace, NodeStyle

from haptk import utils

def normal_tree_style():
    ts = TreeStyle()
    ts.rotation = 90
    ts.show_leaf_name = False
    ts.show_scale = False
    ts.min_leaf_separation = 0
    ts.branch_vertical_margin = 5
    ts.optimal_scale_level = "full"
    ts.allow_face_overlap = True
    return ts


def draw_normal_tree(hst, t, samples_to_tag, colors, proportions, show_haplotype, n_markers, show_pos, branch_point_size, branch_length_as_majority):
    style = NodeStyle()
    style["size"] = 0

    # spent_parents = []
    for n in t.traverse():
        n.set_style(style)

        node_data = hst.get_node_data(int(n.name))

        label = len(node_data["indexes"])

        if proportions:
            top_data = hst.get_node_data(int('0'))
            samplen = len(top_data["indexes"])
            prc = label / samplen
            label = f"N={label} ({prc:.2f})"
        else:
            label = f"{label}"

        if show_haplotype:
            label = f"N={len(node_data['indexes'])}\n{hst.haplotype_string(node_data, n_markers)}"

        if show_pos:
            label = f"N={len(node_data['indexes'])}"
            haplotype = hst.haplotype_string(node_data, n_markers)
            start = node_data['start']['pos']
            stop = node_data['stop']['pos']
            label = f"{label}\n{haplotype}\nstart: {start}\nstop: {stop}"

        F = TextFace(label, tight_text=True, penwidth=30)
        F.rotation = 270
        n.add_face(F, column=0)

        # text_face = TextFace("", fsize=30, tight_text=True, fgcolor="red")
        # text_face.rotation = 270
        # text_face.margin_right = 10
        # utils.tag_branching_point_to_node(hst, n, text_face, branch_point_size)

        if n.is_leaf():
            if samples_to_tag == []:
                n.set_style(utils.return_node_style("#000", 1))

            for (color_i, sample_list) in enumerate(samples_to_tag):
                results = [i for i in node_data["indexes"] if hst.get_sample_name(i) in sample_list]
                if results:
                    n.set_style(utils.return_node_style(colors[color_i], 7 - color_i))

            # text_face = TextFace("", fsize=30, fgcolor="blue")
            # spent_parents = utils.tag_big_branch_to_node(hst, n, spent_parents, text_face, branch_length_as_majority)


    return t




import argparse
import rustworkx as rx

def read_path_to_hst(path):
    with gzip.open(path, 'rt') as f:
        json_bytes = f.read()
        hst_struct = json.loads(json_bytes)
        hst = hst_struct["hst"]
        coords = hst_struct["coords"]
        samples = hst_struct["samples"]
        metadata = hst_struct["metadata"]

        G = rx.PyDiGraph()

        edges_tuple_vec = []
        for edges in hst["edges"]:
            edges_tuple_vec.append((edges[0], edges[1], edges[2]))

        G.add_nodes_from(hst["nodes"])
        G.add_edges_from(edges_tuple_vec)

        if metadata["selection"] == "All":
            samples = [val for val in samples for _ in (0, 1)]

        return G, coords, samples, metadata

### Parameters
parser = argparse.ArgumentParser()
parser.add_argument('--hst', type=str)
parser.add_argument('--match-hst', type=str)

args = parser.parse_args()

# Read in both .hst.gz
HST, coords1, samples1, metadata1 = read_path_to_hst(args.hst)
HST2, coords2, samples2, metadata2 = read_path_to_hst(args.match_hst)

if coords1 != coords2:
    print("coords error")

if samples1 != samples2:
    print("samples error")

if metadata1 != metadata2:
    print("metadata error")

e1 = HST.num_edges()
e2 = HST2.num_edges()

n1 = HST.num_nodes()
n2 = HST2.num_nodes()

print(f"e1: {e1}, e2: {e2}")
print(f"n1: {n1}, n2: {n2}")

el1 = HST.edge_list()
el2 = HST2.edge_list()
nl1 = HST.nodes()
nl2 = HST2.nodes()

emin = min(e1, e2)

for i in range(0, emin):
    if el1[i] != el2[i] or nl1[i] != nl2 [i]:
        print(f"failed equality {i}")
        # print(f"edge  {i}: {el1[i]} != {el2[i]}")
        # print(f"node  {i}: {nl1[i+1]} != {nl2[i+1]}")
        # break
